using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Resources;
using System.IO;
using System.Windows.Forms;
using ZedGraph;


namespace Stairway
{
    public partial class Graphs : Form
    {
        LineItem myCurveFacade;
        LineItem myCurveTraffic;
        
        LineItem myRefCurve;
        LineItem myRefCurveMakeInvisibleLine;
        LineItem myCurveRefs;
        LineItem myShiftedRefCurveMakeInvisibleLine;
        
        LineItem myCurveIndoor;

        GraphPane myPane;
        ZedGraphControl zgc;

        double[] trArr;
        string trString;
        double[] trStdDev;

        double[] faArr;
        double buildings;
        double[] faStdDev;
        
        double[] refs;
        double lekvTr;
        int numFa;

        double[] indoor;

        double[] returns;
        
        ErrorBarItem myCurveFaStdDev;
        //LineItem myCurveFacadeMin;
        //LineItem myCurveFacadeMax;

        ErrorBarItem myCurveTrStdDev;
        
        // traffic data
        int numTr;
        int trSpeedMin;
        int trSpeedMax;
        double[] freqband = new double[] { 50, 63, 80, 100, 125, 160, 200, 250, 315, 400, 500, 630, 800, 1000, 1250, 1600, 2000, 2500, 3150, 4000, 5000 };
        double[] refcurve = { 22, 26, 29, 33, 36, 39, 42, 45, 48, 51, 52, 53, 54, 55, 56, 56, 56, 56, 56, 56, 56 };
        
        int calcMethod;
        double roomVolume;
        double partitionArea;
        MaterialData mat;
        double[] reverberationTime;
        RevData revData;
        Boolean isUsingRevData;
       
        ArrowObj arrow;
        ArrowObj arrow2;
        TextObj text;
        TextObj text2;
        TextObj text3;
        TextObj text4;
        BoxObj box;
        BoxObj box2;

        int sylnr;
        int opm;

        double fc1;
        double fc2;
        double fd;

        string levDiff; 

        public Graphs(double[] trafficArr, double[] trafficStdDev, string trafficString, double[] facadeArr, double[] facadeStdDev, 
            double buildingsCorr, int numberOfFacades, int numberOfTraffics,
            int trSpeed, int trSpeed2, int caMeth, 
            double rVolume, double pArea, MaterialData matData, 
            double[] revTime, RevData rData, Boolean isUsingRevDef, double crit1, double crit2, double mamf,
            int sylnr, int opm)
        {
            this.fc1 = 0;
            this.fc2 = 0;
            this.fd = 0;
            this.opm = 0;
            this.sylnr = 0;

            this.trArr = trafficArr;
            this.trString = "Traffic";
            this.trStdDev = trafficStdDev;
            this.faArr = facadeArr;
            this.buildings = 0.00;
            buildings = buildingsCorr;
            this.faStdDev = facadeStdDev;
            this.numFa = numberOfFacades;
            this.numTr = numberOfTraffics;
            this.trSpeedMin = trSpeed;
            this.trSpeedMax = trSpeed2;
            this.calcMethod = caMeth;
            this.roomVolume = rVolume;
            this.partitionArea = pArea;
            this.mat = matData;
            this.reverberationTime = revTime;
            this.revData = rData;
            this.isUsingRevData = isUsingRevDef;
            this.fc1 = crit1;
            this.fc2 = crit2;
            this.fd = mamf;
            this.sylnr = sylnr;
            this.opm = opm;

            text = null;

            InitializeComponent();
            Console.WriteLine("Graphs1 initialised");

            
            this.indoor = null;

            #region Sets label for Level Diff or SRI
            if (faArr != null)
            {
                // 0 = R
                // 1 = D
                // 2 = Dn
                // 3 = DnT
                // 4 = Dls2m
                if (caMeth == 0)
                {
                   levDiff = "Appartent sound reduction, R'w(C;Ctr;C_(50-5000);Ctr_(50-5000)) = ";
                }
                else if (caMeth == 1)
                {
                    levDiff = "Level difference, Dw(C;Ctr;C_(50-5000);Ctr_(50-5000)) = ";
                }
                else if (caMeth == 2)
                {
                    levDiff = "Normalised level difference, Dn,2m,w(C;Ctr;C_(50-5000);Ctr_(50-5000)) = ";
                }
                else if (caMeth == 3)
                {
                    levDiff = "Standardised level difference, DnT,w(C;Ctr;C_(50-5000);Ctr_(50-5000)) = ";
                }
                else if (caMeth == 4)
                {
                    levDiff = "Standardised level difference, DnT,2m,w(C;Ctr;C_(50-5000);Ctr_(50-5000)) = ";
                }
                else if (caMeth == 9)
                {
                    levDiff = "";
                }
                else 
                {
                    //do nothing
                }
                this.labelLevDiff.Text = levDiff;
            }
#endregion
           
        }


        #region calculates logarithmic sum
        public double sumLevel(double[] arr) 
        {
            double temp = 0.00;
            for (int i = 0; i < arr.Length; i++ )
            {
                temp = temp + Math.Pow(10, (arr[i] / 10));
            }
            temp = 10 * Math.Log10(temp);
            //Console.WriteLine("finding sum: " + temp.ToString());

            return temp;
        }
        #endregion

        #region create chart Main charting method
        public void CreateChart(ZedGraphControl zgc, string titletext, string xaxis, string yaxis )
        {
            //Console.WriteLine("CreateChart(...)" + faArr.GetType() + trArr.GetType());
            this.zgc = zgc;
            myPane = zgc.GraphPane;
            if (trArr != null && faArr != null)
            {
                myPane.Title.Text = titletext;
            }
            else if (trArr != null && faArr == null)
            {
                myPane.Title.Text = "Traffic";
            }
            else if (trArr == null && faArr != null)
            {
                myPane.Title.Text = "Facade";
            }
            else
                myPane.Title.Text = "Empty";

            myPane.XAxis.Title.Text = xaxis;
            myPane.YAxis.Title.Text = yaxis;
            
            string[] ret = new string[] { "50", "63", "80", "100", "125", "160", "200", "250", "315", "400", "500", "630", "800", "1000", "1250", "1600", "2000", "2500", "3150", "4000", "5000" };

            if (trArr != null)
            {
                for (int j = 0; j < trArr.Length; j++) //Finner totalt avvik relativt referansekurven 
                {
                    trArr[j] = trArr[j] +(6 - buildings); //Is this true?
                }

                myCurveTraffic = myPane.AddCurve(trString,
                null, trArr, Color.Blue, ZedGraph.SymbolType.None);
                //smoothing traffick data
                //myCurveTraffic.Line.IsSmooth = true;

                lekvTr = sumLevel(trArr); //adds (6 - building correction) dB for facadecorrection
                this.labelTrafficLevel.Text = "Traffic level at facade (+6 dB): " +  Math.Round(lekvTr,1).ToString() + " dB(A)";    
            }

            if (numTr != 0)
            {
                //kanskje ha med noe refleksjonsforhold?
                this.labelNumTr.Visible = true;
                this.labelNumTr.Text = "Number of Traffic measurments: " + this.numTr.ToString();
            }
            else
            {
                this.labelNumTr.Visible = false;
            }

            //Calculating facade curve according to set method
            //Data from Stairway
            #region Facade Data from StairWay
            if ((faArr != null) && (numFa != 0))
            {
                Console.WriteLine("Correcting data from StairWay");

                //Checks if reverberation time is set
                if (this.isUsingRevData == true)
                {
                    reverberationTime = revData.getRevData();
                }
                                
                //Checks calculating method
                if (calcMethod == 0)
                {
                    //R'
                    Console.WriteLine("Calculating R according to method, faArr[j]" + faArr[2].ToString()
                        +" mat.aSpeed"+ mat.aSpeed.ToString() + "roomvolume" + roomVolume.ToString() +
                        " rev time: " + reverberationTime.ToString() + " PartitionArea:" +
                        partitionArea.ToString());
                       
                    for (int j = 0; j < faArr.Length; j++) //Finner totalt avvik relativt referansekurven 
                    {
                        double A = (24 * Math.Log(10) / mat.aSpeed ) * roomVolume / reverberationTime[j];
                        this.faArr[j] = faArr[j] + 10 * Math.Log10(partitionArea / A) - 3;
                    }
                    
                }
                else if (calcMethod == 1)
                {
                    //D do nothing
                    Console.WriteLine("Calculating D");
                }
                else if (calcMethod == 2)
                {
                    //%Dn2m trukket fra 3dB p nivdifferansen i ligningen under for  f lydnivet 2meter fra veggen!!!
                    Console.WriteLine("Calculating DnT");
                    for (int j = 0; j < faArr.Length; j++) //Finner totalt avvik relativt referansekurven 
                    {
                        double A = (24 * Math.Log(10) / mat.aSpeed ) * roomVolume / reverberationTime[j];
                        double A_0 = 10;
                        faArr[j] = faArr[j] - 3 - 10 * Math.Log10(A / A_0);
                    }
                }
                else if (calcMethod == 3)
                {
                    //DnT
                    Console.WriteLine("Calculating Dls2m");
                    for (int j = 0; j < faArr.Length; j++) //Finner totalt avvik relativt referansekurven 
                    {
                        double T_0 = 0.5;
                        faArr[j] = faArr[j] + 10 * Math.Log10(reverberationTime[j] / T_0);
                    }
                }
                else if (calcMethod == 4)
                {
                    //Dls2m
                    for (int j = 0; j < faArr.Length; j++) //Finner totalt avvik relativt referansekurven 
                    {
                        double T_0 = 0.5;
                        faArr[j] = faArr[j] + 10 * Math.Log10(reverberationTime[j] / T_0) -3;
                    }
                }
                else 
                {
                }

                int index = levDiff.IndexOf(",");
                myCurveFacade = myPane.AddCurve(levDiff.Remove(index).Trim(),
                null, faArr, Color.Red, ZedGraph.SymbolType.None);
            }
            #endregion


            //Data from Calculation
            #region Facade Data from Calculations
            if ((faArr != null) && (numFa == 0))
            {
                Console.WriteLine("Correcting data from calculations");
                //Checks if reverberationTime is set
                if (this.isUsingRevData == true)
                {
                    reverberationTime = revData.getRevData();
                }
                
                //Checks calculating method
                if (calcMethod == 0)
                {
                    //R 
                    Console.WriteLine("Calculating ");
                    for (int j = 0; j < faArr.Length; j++) //Finner totalt avvik relativt referansekurven 
                    {
                        double A = (24 * Math.Log(10) / mat.aSpeed) * roomVolume / reverberationTime[j];
                        faArr[j] = faArr[j] + 10 * Math.Log10(partitionArea / A);
                    }
                }
                else if (calcMethod == 1)
                {
                    //D 
                    Console.WriteLine("Calculating D");
                    for (int j = 0; j < faArr.Length; j++) //Finner totalt avvik relativt referansekurven 
                    {
                        double A = (24 * Math.Log(10) / mat.aSpeed) * roomVolume / reverberationTime[j];
                        faArr[j] = faArr[j] - 10 * Math.Log10(partitionArea / A) + 3;
                    }
                }
                else if (calcMethod == 2)
                {
                    //%Dn trukket fra 3dB p nivdifferansen i ligningen under for  f lydnivet 2meter fra veggen!!!
                    Console.WriteLine("Calculating DnT");
                    for (int j = 0; j < faArr.Length; j++) //Finner totalt avvik relativt referansekurven 
                    {
                        double A = (24 * Math.Log(10) / mat.aSpeed) * roomVolume / reverberationTime[j];
                        faArr[j] = faArr[j] - 10 * Math.Log10(partitionArea / A) + 3;
                        double A_0 = 10;
                        faArr[j] = faArr[j] - 3 - 10 * Math.Log10(A / A_0);
                    }
                }
                else if (calcMethod == 3)
                {
                    //DnT
                    Console.WriteLine("Calculating Dls2m");
                    for (int j = 0; j < faArr.Length; j++) //Finner totalt avvik relativt referansekurven 
                    {
                        double A = (24 * Math.Log(10) / mat.aSpeed) * roomVolume / reverberationTime[j];
                        faArr[j] = faArr[j] - 10 * Math.Log10(partitionArea / A) + 3;
                        double T_0 = 0.5;
                        faArr[j] = faArr[j] + 10 * Math.Log10(reverberationTime[j] / T_0);
                    }
                }
                else if (calcMethod == 4)
                {
                    //Dls2m
                    for (int j = 0; j < faArr.Length; j++) //Finner totalt avvik relativt referansekurven 
                    {
                        double A = (24 * Math.Log(10) / mat.aSpeed) * roomVolume / reverberationTime[j];
                        faArr[j] = faArr[j] - 10 * Math.Log10(partitionArea / A) + 3;
                        double T_0 = 0.5;
                        faArr[j] = faArr[j] + 10 * Math.Log10(reverberationTime[j] / T_0) - 3;
                    }
                }
                else
                {
                }

                int index = levDiff.IndexOf(",");
                myCurveFacade = myPane.AddCurve(levDiff.Remove(index).Trim(),
                null, faArr, Color.Red, ZedGraph.SymbolType.None);
            }
            #endregion
            

            if (numFa != 0)
            {
                this.labelNumFa.Visible = true;
                this.labelNumFa.Text = "Number of Facade measurments: " + this.numFa.ToString();
            }
            else
            {
                this.labelNumFa.Visible = false;
            }

            Console.WriteLine("Writing the helplines!!");
            //Draws two helplines at 100 and 3150 Hz
            LineObj myLine100 = new LineObj(Color.Black, 4, -10000, 4, 10000);
            myLine100.Style = System.Drawing.Drawing2D.DashStyle.Dash;
            myPane.GraphObjList.Add(myLine100);
            myLine100.IsVisible = true;
            myLine100.IsClippedToChartRect = true;

            LineObj myLine3150 = new LineObj(Color.Black, 19, -10000, 19, 10000);
            myLine3150.Style = System.Drawing.Drawing2D.DashStyle.Dash;
            myPane.GraphObjList.Add(myLine3150);
            myLine3150.IsVisible = true;
            myLine3150.IsClippedToChartRect = true;
            //LineItem refmax = myPane.AddCurve(null,
            //null, faArr, Color.Red, ZedGraph.SymbolType.None);


            myPane.XAxis.Type = AxisType.Text;
            myPane.XAxis.MinorGrid.IsVisible = true;
            myPane.XAxis.Scale.TextLabels = ret;
            myPane.XAxis.MajorGrid.Color = Color.LightGray;
            myPane.Legend.Position = ZedGraph.LegendPos.Right;
            //myPane.Chart.Fill = new Fill(Color.White, Color.SteelBlue,45.0F );
            //myPane.Fill = new Fill(Color.WhiteSmoke, Color.Lavender, 12F);

            /**
            BoxObj box = new BoxObj(50, 50, 80, 300, Color.Empty,Color.FromArgb(150, Color.LightGreen));
            box.Fill = new Fill(Color.White, Color.FromArgb(200, Color.LightGreen), 45.0F);
            // Use the BehindAxis zorder to draw the highlight beneath the grid lines
            box.ZOrder = ZOrder.E_BehindAxis;
            myPane.GraphObjList.Add(box);
           **/
                 
            // Calculate the Axis Scale Ranges
            zgc.AxisChange();

            this.Show();
            this.Refresh();
        }
        #endregion

        private void Graphs_Load(object sender, EventArgs e)
        { 
            CreateChart(this.zedGraphControl1, "Traffic vs Facade spectrum ", "Frequency [Hz]", "Magnitude [dB]");
        }

        #region GUI-alternatives
        private void checkBoxAverage_CheckedChanged(object sender, EventArgs e)
        {
            if (this.checkBoxAverage.Checked == true)
            {
                this.checkBoxRefCurve.Enabled = true;
                this.checkBoxShiftedRefCurve.Enabled = true;
            }
            else 
            {
                this.checkBoxRefCurve.Enabled = false;
                this.checkBoxRefCurve.Checked = false;
                this.checkBoxShiftedRefCurve.Enabled = false;
                this.checkBoxShiftedRefCurve.Checked = false;
                this.labelLevDiff.Text = "Results:";
                this.labelMaxUnfavDev.Text = "";
                this.labelUnfavDev.Text = "";
            }
        }
        #endregion
        
        #region Calculates weighted @500Hz from ISO 717-1 from the facade data
        private double[] calcW(double[] p)
        {
            returns = new double[] { 0, 0, 0, 0, 0, 0, 0, 0, 0 };
            this.refs = new double[]{ 22, 26, 29, 33, 36, 39, 42, 45, 48, 51, 52, 53, 54, 55, 56, 56, 56, 56, 56, 56, 56 };

            double[] cRefCurve = new double[] { -40, -36, -33, -29, -26, -23, -21, -19, -17, -15, -13, -12, -11, -10, -9, -9, -9, -9, -9, -9, -9 };
            double[] ctrRefCurve = new double[] { -25, -23, -21, -20, -20, -18, -16, -15, -14, -13, -12, -11, -9, -8, -9, -10, -11, -13, -15, -16, -18 };
           
            double maxUnfavDev = 0;
            int pw = 0;
            double avvik = 0;
            int c = 0;
            int maxAvvikFreq = 0;

            double CTemp = 0;
            double CTempExt = 0;
            double CtrTemp = 0;
            double CtrTempExt = 0;

            for (int j = 3; j < 19; j++) //Finner totalt avvik relativt referansekurven 
            {
                if (p[j] < refcurve[j])
                {
                    avvik = avvik + (refcurve[j] - p[j]);
                    //Console.WriteLine("total avvik:" + avvik + " refcurve: " +refcurve[j]);
                }
            }

            if (Math.Round(avvik) > 32) //Dersom avvik er over 32 dB
            {
                c = 0;
                while (Math.Round(avvik) > 32) //Senker referansekurve til avvik er lik eller under 32 dB
                {
                    CTemp = 0;
                    CTempExt = 0;
                    CtrTemp = 0;
                    CtrTempExt = 0;
                    avvik = 0;
                    c = c + 1;
                    maxUnfavDev = 0;
                    maxAvvikFreq = 0;
                    for (int j = 3; j < 19; j++) 
                    {
                        if (p[j] < (refcurve[j] - c))
                        {
                            avvik = avvik + refcurve[j] - c - p[j];
                        }
                        if (maxUnfavDev < (refcurve[j] - c - p[j]))
                        {
                            maxUnfavDev = (refcurve[j] - c - p[j]);
                            maxAvvikFreq = j;
                            //Console.WriteLine("Unfav deviations maxen: " + maxUnfavDev.ToString());
                        }
                        //Calculating C and Ctr 100 - 3150Hz
                        CTemp = CTemp + Math.Pow(10, ((cRefCurve[j] - p[j]) / 10));
                        CtrTemp = CtrTemp + Math.Pow(10, ((ctrRefCurve[j] - p[j]) / 10));

                        //Calculating Ctr 50 - 5000Hz
                        if (j == 3) 
                        {
                            CtrTempExt = CtrTempExt + Math.Pow(10, ((ctrRefCurve[j-3] - p[j-3]) / 10));
                            CtrTempExt = CtrTempExt + Math.Pow(10, ((ctrRefCurve[j-2] - p[j-2]) / 10));
                            CtrTempExt = CtrTempExt + Math.Pow(10, ((ctrRefCurve[j-1] - p[j-1]) / 10));
                            CtrTempExt = CtrTempExt + Math.Pow(10, ((ctrRefCurve[j] - p[j]) / 10));
                        }
                        else if (j == 18)
                        {
                            CtrTempExt = CtrTempExt + Math.Pow(10, ((ctrRefCurve[j+2] - p[j+2]) / 10));
                            CtrTempExt = CtrTempExt + Math.Pow(10, ((ctrRefCurve[j+1] - p[j+1]) / 10));
                            CtrTempExt = CtrTempExt + Math.Pow(10, ((ctrRefCurve[j] - p[j]) / 10));
                        }
                        else 
                        {
                            CtrTempExt = CtrTempExt + Math.Pow(10, ((ctrRefCurve[j] - p[j]) / 10));
                        }

                        //Calculating C 50 - 5000Hz
                        if (j == 3) 
                        {
                            CTempExt = CTempExt + Math.Pow(10, ((cRefCurve[j-3] - 1 - p[j-3]) / 10));
                            CTempExt = CTempExt + Math.Pow(10, ((cRefCurve[j-2] - 1 - p[j-2]) / 10));
                            CTempExt = CTempExt + Math.Pow(10, ((cRefCurve[j-1] - 1- p[j-1]) / 10));
                            CTempExt = CTempExt + Math.Pow(10, ((cRefCurve[j] - 1 - p[j]) / 10));
                        }
                        else if (j == 18)
                        {
                            CTempExt = CTempExt + Math.Pow(10, ((cRefCurve[j + 2] - 1 - p[j + 2]) / 10));
                            CTempExt = CTempExt + Math.Pow(10, ((cRefCurve[j + 1] - 1 - p[j + 1]) / 10));
                            CTempExt = CTempExt + Math.Pow(10, ((cRefCurve[j] - 1 - p[j]) / 10));
                        }
                        else
                        {
                            CTempExt = CTempExt + Math.Pow(10, ((cRefCurve[j] - 1 - p[j]) / 10));
                        }
                    }
                }
                pw = 52 - c;
            }

            else if (Math.Round(avvik) < 32) //Dersom avvik er under 32 dB
            {
                c = 0;
                while (Math.Round(avvik) < 32) //Hever referanskurve til avvik er over 32 dB
                {
                    CTemp = 0;
                    CTempExt = 0;
                    CtrTemp = 0;
                    CtrTempExt = 0;
                    avvik = 0;
                    c = c - 1;
                    maxUnfavDev = 0;
                    maxAvvikFreq = 0;
                    for (int j = 3; j < 19; j++)
                    {
                        if (p[j] < (refcurve[j] - c))
                        {
                            avvik = avvik + refcurve[j] - c - p[j];
                        }
                        if (maxUnfavDev < (refcurve[j] - c - p[j]))
                        {
                            maxUnfavDev = (refcurve[j] - c - p[j]);
                            maxAvvikFreq = j;
                            //Console.WriteLine("Unfav deviations maxen: " + maxUnfavDev.ToString());
                        }
                        //Calculating C and Ctr from 100-3150 Hz
                        CTemp = CTemp + Math.Pow(10, ((cRefCurve[j] - p[j]) / 10));
                        CtrTemp = CtrTemp + Math.Pow(10, ((ctrRefCurve[j] - p[j]) / 10));
                        
                        //Calculating Ctr 50 - 5000Hz
                        if (j == 3)
                        {
                            CtrTempExt = CtrTempExt + Math.Pow(10, ((ctrRefCurve[j - 3] - p[j - 3]) / 10));
                            CtrTempExt = CtrTempExt + Math.Pow(10, ((ctrRefCurve[j - 2] - p[j - 2]) / 10));
                            CtrTempExt = CtrTempExt + Math.Pow(10, ((ctrRefCurve[j - 1] - p[j - 1]) / 10));
                            CtrTempExt = CtrTempExt + Math.Pow(10, ((ctrRefCurve[j] - p[j]) / 10));
                        }
                        else if (j == 18)
                        {
                            CtrTempExt = CtrTempExt + Math.Pow(10, ((ctrRefCurve[j + 2] - p[j + 2]) / 10));
                            CtrTempExt = CtrTempExt + Math.Pow(10, ((ctrRefCurve[j + 1] - p[j + 1]) / 10));
                            CtrTempExt = CtrTempExt + Math.Pow(10, ((ctrRefCurve[j] - p[j]) / 10));
                        }
                        else
                        {
                            CtrTempExt = CtrTempExt + Math.Pow(10, ((ctrRefCurve[j] - p[j]) / 10));
                        }

                        //Calculating C 50 - 5000Hz
                        if (j == 3)
                        {
                            CTempExt = CTempExt + Math.Pow(10, ((cRefCurve[j - 3] - 1 - p[j - 3]) / 10));
                            CTempExt = CTempExt + Math.Pow(10, ((cRefCurve[j - 2] - 1 - p[j - 2]) / 10));
                            CTempExt = CTempExt + Math.Pow(10, ((cRefCurve[j - 1] - 1 - p[j - 1]) / 10));
                            CTempExt = CTempExt + Math.Pow(10, ((cRefCurve[j] - 1 - p[j]) / 10));
                        }
                        else if (j == 18)
                        {
                            CTempExt = CTempExt + Math.Pow(10, ((cRefCurve[j + 2] - 1 - p[j + 2]) / 10));
                            CTempExt = CTempExt + Math.Pow(10, ((cRefCurve[j + 1] - 1 - p[j + 1]) / 10));
                            CTempExt = CTempExt + Math.Pow(10, ((cRefCurve[j] - 1 - p[j]) / 10));
                        }
                        else
                        {
                            CTempExt = CTempExt + Math.Pow(10, ((cRefCurve[j] - 1 - p[j]) / 10));
                        }
                    }
                }
                if (avvik > 32) // Dersom avvik ble over 32 dB, m referanse kurve senkes 1 dB
                {
                    c = c + 1;
 
                    //Adjusting UnfavDev(avvik) and UnFavDevMax as well
                    maxUnfavDev = 0;
                    avvik = 0;
                    for (int j = 3; j < 19; j++)
                    {
                        if (p[j] < (refcurve[j] - c))
                        {
                            avvik = avvik + refcurve[j] - c - p[j];
                        }
                        if (maxUnfavDev < (refcurve[j] - c - p[j]))
                        {
                            maxUnfavDev = (refcurve[j] - c - p[j]);
                            maxAvvikFreq = j;
                        }
                    }
                    //Console.WriteLine("writing dw, avvik:" + avvik);
                }
                pw = 52 - c;
            }
            //Console.WriteLine("writing dw, avvik:" + avvik);

            for (int j = 0; j < refcurve.Length; j++)
            {
                refs[j] = refcurve[j] - c;
            }

            Console.WriteLine("wrinting c: " + CTemp.ToString() + " Ctr: " + CtrTemp.ToString() + " CtrExt: " + CtrTempExt.ToString() +
                 "CTempExt: " + CTempExt.ToString());

            CTemp = Convert.ToInt32(Math.Round(CTemp - 10*(Math.Log10(CTemp)) - pw,0));
            CTempExt = Convert.ToInt32(Math.Round(CTempExt - 10 * (Math.Log10(CTempExt)) - pw, 0));
            CtrTemp = Convert.ToInt32(Math.Round(CtrTemp - 10*(Math.Log10(CtrTemp)) - pw,0));
            CtrTempExt = Convert.ToInt32(Math.Round(CtrTempExt - 10*(Math.Log10(CtrTempExt)) - pw,0));
           
            Console.WriteLine("wrinting c: " + CTemp.ToString() + " Ctr: " + CtrTemp.ToString()+ " CtrExt: " + CtrTempExt.ToString() +
                 "CTempExt: " + CTempExt.ToString());

            //Console.WriteLine("Calculations done! Avvik: " + avvik.ToString());
            returns[0] = pw;
            returns[1] = avvik;
            returns[2] = maxUnfavDev;
            returns[3] = CTemp;
            returns[4] = CtrTemp;
            returns[5] = CTempExt;
            returns[6] = CtrTempExt;
            returns[7] = maxAvvikFreq;
           
            //Checks reductionnumer or leveldifference
            if (true)
            {
                returns[8] = 1; //1 = Dw
            }
                        
            return returns;
        }
        #endregion

        #region Draws the refcurve on the pane
        private void checkBoxRefCurve_CheckedChanged(object sender, EventArgs e)
        {
            
            if (this.checkBoxRefCurve.Checked == true && this.faArr != null ) 
            {
                double[] tempRes = calcW(faArr);
                int temporary = Convert.ToInt32(tempRes.GetValue(0));
                double avvikTemp = Convert.ToDouble(tempRes.GetValue(1));
                double maxUnfavDevTemp = Convert.ToDouble(tempRes.GetValue(2));
                double CCorrection = Convert.ToDouble(tempRes.GetValue(3));
                double CtrCorrection = Convert.ToDouble(tempRes.GetValue(4));
                double CExtCorrection = Convert.ToDouble(tempRes.GetValue(5));
                double CtrExtCorrection = Convert.ToDouble(tempRes.GetValue(6));
                string maxAvvikFreq = freqband[Convert.ToInt32(tempRes.GetValue(7))].ToString();

                this.labelUnfavDev.Text = "Sum of unfavorabile deviations: " + Math.Round(avvikTemp, 1).ToString() + " dB";
                this.labelMaxUnfavDev.Text = "Maximum unfavorable deviation: " + Math.Round(maxUnfavDevTemp, 1).ToString() + 
                    " dB at " + maxAvvikFreq.ToString() + " Hz";
                //this.labelUnFavDevR.Text = "Sum of unfavorabile deviations: " + Math.Round(avvik, 1).ToString() + " dB";
                //this.labelMaxUnfavDevR.Text = "Maximum unfavorable deviation: " + Math.Round(maxUnfavDev, 1).ToString() + " dB";
            
                double[] myRefCurveMakeInvisible = { 22, 26, 29, 33, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 56, 56, 56 };
                myRefCurveMakeInvisibleLine = myPane.AddCurve(null, null, myRefCurveMakeInvisible, Color.White, ZedGraph.SymbolType.None);
                myRefCurve = myPane.AddCurve("Ref curve", null, refcurve, Color.Orange, ZedGraph.SymbolType.None);
                //myRefCurve.Line.Style = System.Drawing.Drawing2D.DashStyle.Dot;

                
                this.labelLevDiff.Text = levDiff + temporary.ToString() + "(" +
                   CCorrection.ToString() + ";" + CtrCorrection.ToString() + ";" +
                   CExtCorrection.ToString() + ";" + CtrExtCorrection.ToString() + ") dB";

                zgc.AxisChange();
                this.Show();
                this.Refresh();
            }

            else if (faArr == null)
            {
                MessageBox.Show("Facade data missing");
                this.checkBoxIndoorLevel.Checked = false;
            }

            else 
            {
                myRefCurve.Label.IsVisible = false;
                myRefCurveMakeInvisibleLine.IsVisible = false;
                myRefCurveMakeInvisibleLine.Clear();
                myRefCurve.Clear();
                zgc.AxisChange();
                this.Show();
                this.Refresh();
            }

        }
        #endregion 

        #region Calculates indoor level
        private void checkBoxIndoorLevel_CheckedChanged(object sender, EventArgs e)
        {
            if (faArr != null && trArr != null && this.checkBoxIndoorLevel.Checked == true )
            {
                /**
                //Checks if data is under 10 meter square
                if (this.partitionArea < 10)
                {
                    partitionArea = 10;
                    MessageBox.Show("Partition area set to 10 m^2");
                }
                 * **/

                double[] tempTrArr = new double[trArr.Length];
                this.indoor = new double[trArr.Length];
                
                tempTrArr = trArr;
                if (this.checkBoxAweight.Checked == false)
                {
                    tempTrArr = this.doAweight(trArr);
                }


               // indoor = new double[tempTrArr.Length];

                #region Facade Data from StairWay
                if ((faArr != null) && (numFa != 0))
                {
                    Console.WriteLine("Correcting data from StairWay");

                    //Checks if reverberation time is set
                    if (this.isUsingRevData == true)
                    {
                        reverberationTime = revData.getRevData();
                    }

                    //Checks calculating method
                    if (calcMethod == 0)
                    {
                        //R'
                        Console.WriteLine("Calculating R according to method, faArr[j]" + faArr[2].ToString()
                            + " mat.aSpeed" + mat.aSpeed.ToString() + "roomvolume" + roomVolume.ToString() +
                            " rev time: " + reverberationTime.ToString() + " PartitionArea:" +
                            partitionArea.ToString());

                        for (int j = 0; j < faArr.Length; j++) //Finner totalt avvik relativt referansekurven 
                        {
                            double A = (24 * Math.Log(10) / mat.aSpeed) * roomVolume / reverberationTime[j];
                            this.faArr[j] = faArr[j] - ( 10 * Math.Log10(partitionArea / A) - 3);
                            
                                                    
                            this.indoor[j] = tempTrArr[j] - faArr[j] + 10 * Math.Log10(partitionArea/10) +
                               10 * Math.Log10(reverberationTime[j]/0.5) - 10 * Math.Log10(roomVolume/31) - 3;
                            //Console.WriteLine("Calculating R 2:" + this.sumLevel(indoor).ToString());
                        }

                    }
                    else if (calcMethod == 1)
                    {
                        //D do nothing
                        Console.WriteLine("Calculating D");
                        for (int j = 0; j < faArr.Length; j++) //Finner totalt avvik relativt referansekurven 
                        {
                            this.indoor[j] = tempTrArr[j] - faArr[j] + 10 * Math.Log10(partitionArea / 10) +
                              10 * Math.Log10(reverberationTime[j] / 0.5) - 10 * Math.Log10(roomVolume / 31) - 3;
                        }
                    }
                    else if (calcMethod == 2)
                    {
                        //%Dn2m trukket fra 3dB p nivdifferansen i ligningen under for  f lydnivet 2meter fra veggen!!!
                        Console.WriteLine("Calculating DnT");
                        for (int j = 0; j < faArr.Length; j++) //Finner totalt avvik relativt referansekurven 
                        {
                            double A = (24 * Math.Log(10) / mat.aSpeed) * roomVolume / reverberationTime[j];
                            double A_0 = 10;
                            faArr[j] = faArr[j] + 3 + 10 * Math.Log10(A / A_0);
                            this.indoor[j] = tempTrArr[j] - faArr[j] + 10 * Math.Log10(partitionArea / 10) +
                              10 * Math.Log10(reverberationTime[j] / 0.5) - 10 * Math.Log10(roomVolume / 31) - 3;
                        }
                    }
                    else if (calcMethod == 3)
                    {
                        //DnT
                        Console.WriteLine("Calculating DnT");
                        for (int j = 0; j < faArr.Length; j++) //Finner totalt avvik relativt referansekurven 
                        {
                            double T_0 = 0.5;
                            faArr[j] = faArr[j] - 10 * Math.Log10(reverberationTime[j] / T_0);
                            this.indoor[j] = tempTrArr[j] - faArr[j] + 10 * Math.Log10(partitionArea / 10) +
                             10 * Math.Log10(reverberationTime[j] / 0.5) - 10 * Math.Log10(roomVolume / 31) - 3;
                        }
                    }
                    else if (calcMethod == 4)
                    {
                        //Dls2m
                        Console.WriteLine("Calculating Dls2m");
                        for (int j = 0; j < faArr.Length; j++) //Finner totalt avvik relativt referansekurven 
                        {
                            double T_0 = 0.5;
                            faArr[j] = faArr[j] - 10 * Math.Log10(reverberationTime[j] / T_0) + 3;
                            this.indoor[j] = tempTrArr[j] - faArr[j] + 10 * Math.Log10(partitionArea / 10) +
                              10 * Math.Log10(reverberationTime[j] / 0.5) - 10 * Math.Log10(roomVolume / 31) - 3;
                        }
                    }
                    else
                    {
                    }

                    
                }
                #endregion

                #region Facade Data from Calculations
                if ((faArr != null) && (numFa == 0))
                {
                    Console.WriteLine("Correcting data from calculations");
                    //Checks if reverberationTime is set
                    if (this.isUsingRevData == true)
                    {
                        reverberationTime = revData.getRevData();
                    }

                    //Checks calculating method
                    if (calcMethod == 0)
                    {
                        //R'
                        //do nothing
                        Console.WriteLine("Calculating R");
                        for (int j = 0; j < faArr.Length; j++) //Finner totalt avvik relativt referansekurven 
                        {
                            this.indoor[j] = tempTrArr[j] - faArr[j] + 10 * Math.Log10(partitionArea / 10) +
                             10 * Math.Log10(reverberationTime[j] / 0.5) - 10 * Math.Log10(roomVolume / 31) - 3;
                        }
                    }
                    else if (calcMethod == 1)
                    {
                        //D 
                        Console.WriteLine("Calculating D");
                        for (int j = 0; j < faArr.Length; j++) //Finner totalt avvik relativt referansekurven 
                        {
                            double A = (24 * Math.Log(10) / mat.aSpeed) * roomVolume / reverberationTime[j];
                            faArr[j] = faArr[j] + 10 * Math.Log10(partitionArea / A) - 3;
                            this.indoor[j] = tempTrArr[j] - faArr[j] + 10 * Math.Log10(partitionArea / 10) +
                             10 * Math.Log10(reverberationTime[j] / 0.5) - 10 * Math.Log10(roomVolume / 31) - 3;
                        }
                    }
                    else if (calcMethod == 2)
                    {
                         //%Dn trukket fra 3dB p nivdifferansen i ligningen under for  f lydnivet 2meter fra veggen!!!
                    Console.WriteLine("Calculating indoor level by D");
                    for (int j = 0; j < faArr.Length; j++) //Finner totalt avvik relativt referansekurven 
                    {
                        double A = (24 * Math.Log(10) / mat.aSpeed) * roomVolume / reverberationTime[j];
                        faArr[j] = faArr[j] -(- 10 * Math.Log10(partitionArea / A) + 3);
                        double A_0 = 10;
                        faArr[j] = faArr[j] - (- 3 - 10 * Math.Log10(A / A_0));
                        this.indoor[j] = tempTrArr[j] - faArr[j] + 10 * Math.Log10(partitionArea / 10) +
                             10 * Math.Log10(reverberationTime[j] / 0.5) - 10 * Math.Log10(roomVolume / 31) - 3;
                    }
                        
                    }
                    else if (calcMethod == 3)
                {
                    //DnT
                    Console.WriteLine("Calculating Dls2m");
                    for (int j = 0; j < faArr.Length; j++) //Finner totalt avvik relativt referansekurven 
                    {
                        double A = (24 * Math.Log(10) / mat.aSpeed) * roomVolume / reverberationTime[j];
                        faArr[j] = faArr[j] + 10 * Math.Log10(partitionArea / A) - 3;
                        double T_0 = 0.5;
                        faArr[j] = faArr[j] - 10 * Math.Log10(reverberationTime[j] / T_0);
                        this.indoor[j] = tempTrArr[j] - faArr[j] + 10 * Math.Log10(partitionArea / 10) +
                             10 * Math.Log10(reverberationTime[j] / 0.5) - 10 * Math.Log10(roomVolume / 31) - 3; 
                    }
                }
                else if (calcMethod == 4)
                {
                    //Dls2m
                    for (int j = 0; j < faArr.Length; j++) //Finner totalt avvik relativt referansekurven 
                    {
                        double A = (24 * Math.Log(10) / mat.aSpeed) * roomVolume / reverberationTime[j];
                        faArr[j] = faArr[j] + 10 * Math.Log10(partitionArea / A) + 3;
                        double T_0 = 0.5;
                        faArr[j] = faArr[j] - 10 * Math.Log10(reverberationTime[j] / T_0) - 3;
                        this.indoor[j] = tempTrArr[j] - faArr[j] + 10 * Math.Log10(partitionArea / 10) +
                             10 * Math.Log10(reverberationTime[j] / 0.5) - 10 * Math.Log10(roomVolume / 31) - 3; 
                    }
                }
                    
                    else
                    {
                    }

                    
                }
                #endregion

                myCurveIndoor = myPane.AddCurve("Indoor level",
                    null, indoor, Color.DarkOrchid, ZedGraph.SymbolType.None);
                zgc.AxisChange();
                this.Show();
                this.Refresh();

                double lekvFa = 0.00;
                lekvFa = sumLevel(indoor);
                this.labelIndoorLevel.Text = "Indoor Level, L_ekv,A,24h: " + Math.Round(lekvFa, 1).ToString();
                

            }
            else if (faArr == null || trArr == null)
            {
                MessageBox.Show("Traffic or Facade data missing");
                this.checkBoxIndoorLevel.Checked = false;
            }
            else
            {
                myCurveIndoor.Label.IsVisible = false;
                myCurveIndoor.Clear();
                this.labelIndoorLevel.Text = "Indoor Level, L_ekv,A,24h: ";
                indoor = null;

                zgc.AxisChange();
                this.Show();
                this.Refresh();
            }
        }
        #endregion

        #region draws the shifted reference curve on the graph
        private void checkBoxShiftedRefCurve_CheckedChanged(object sender, EventArgs e)
        {
            if (faArr != null && this.checkBoxAverage.Checked == true && this.checkBoxShiftedRefCurve.Checked == true)
            {
                double[] tempRes = calcW(faArr);
                int temporary = Convert.ToInt32(tempRes.GetValue(0));
                double avvikTemp = Convert.ToDouble(tempRes.GetValue(1));
                double maxUnfavDevTemp = Convert.ToDouble(tempRes.GetValue(2));
                double CCorrection = Convert.ToDouble(tempRes.GetValue(3));
                double CtrCorrection = Convert.ToDouble(tempRes.GetValue(4));
                double CExtCorrection = Convert.ToDouble(tempRes.GetValue(5));
                double CtrExtCorrection = Convert.ToDouble(tempRes.GetValue(6));
                string maxAvvikFreq = freqband[Convert.ToInt32(tempRes.GetValue(7))].ToString();

                this.labelUnfavDev.Text = "Sum of unfavorabile deviations: " + Math.Round(avvikTemp, 1).ToString() + " dB";
                this.labelMaxUnfavDev.Text = "Maximum unfavorable deviation: " + Math.Round(maxUnfavDevTemp, 1).ToString() + 
                    " dB at " + maxAvvikFreq.ToString() + " Hz"; 
                //this.labelUnFavDevR.Text = "Sum of unfavorabile deviations: " + Math.Round(avvik, 1).ToString() + " dB";
                //this.labelMaxUnfavDevR.Text = "Maximum unfavorable deviation: " + Math.Round(maxUnfavDev, 1).ToString() + " dB";
            

                double[] myShiftedRefCurveMakeInvisible = { refs[0], refs[1], refs[2], refs[3], 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, 29, refs[18], refs[19], refs[20] };
                myShiftedRefCurveMakeInvisibleLine = myPane.AddCurve(null, null, myShiftedRefCurveMakeInvisible, Color.White, ZedGraph.SymbolType.None);

                myCurveRefs = myPane.AddCurve("Shifted referance curve", null, refs, Color.Orange, ZedGraph.SymbolType.None);
                //myCurveRefs.Line.Style = System.Drawing.Drawing2D.DashStyle.Dot;
                this.labelLevDiff.Text = levDiff + temporary.ToString() + "(" +
                   CCorrection.ToString() + ";" + CtrCorrection.ToString() + ";" + 
                   CExtCorrection.ToString() + ";" + CtrExtCorrection.ToString() + ") dB";

                zgc.AxisChange();
                this.Show();
                this.Refresh();
            }
            else if (faArr == null)
            {
                MessageBox.Show("Facade data missing");
                this.checkBoxIndoorLevel.Checked = false;
            }
            else
            {
                myCurveRefs.Label.IsVisible = false;
                myCurveRefs.Clear();
                myShiftedRefCurveMakeInvisibleLine.IsVisible = false;
                myShiftedRefCurveMakeInvisibleLine.Clear();
                zgc.AxisChange();
                this.Show();
                this.Refresh();
            }
        }
        #endregion

        #region calculates the a-weigted level
        public double[] doAweight(double[] spc)
        {
            double[] temp = new double[spc.Length];
            double[] aweight = new double[] {-30.2, -26.2, -22.5,
                -19.1, -16.1, -13.4, -10.9, -8.6, -6.6, -4.8, -3.2, -1.9, -0.8,
                0.0, 0.6, 1.0, 1.2, 1.3, 1.2, 1.0, 0.5, -0.1, -1.1, -2.5, -4.3, -6.6, -9.3 };

            for (int i = 0; i < spc.Length; i++)
            {
                temp[i] = spc[i] + aweight[i];
                //Console.WriteLine("writing the spc:" + spc[i]);
            }
            return temp;
        }
        #endregion

        #region calculates the inverse a-weigted level
        public double[] doInverseAweight(double[] spc)
        {
            double[] temp = new double[spc.Length];
            double[] aweight = new double[] {30.2, 26.2, 22.5,
                19.1, 16.1, 13.4, 10.9, 8.6, 6.6, 4.8, 3.2, 1.9, 0.8,
                0.0, -0.6, -1.0, -1.2, -1.3, -1.2, -1.0, -0.5, 0.1, 1.1, 2.5, 4.3, 6.6, 9.3 };

            for (int i = 0; i < spc.Length; i++)
            {
                temp[i] = spc[i] + aweight[i];
                //Console.WriteLine("writing the spc:" + spc[i]);
            }
            return temp;
        }
        #endregion

        #region Aweighing the curves
        private void checkBoxAweight_CheckedChanged(object sender, EventArgs e)
        {
            #region Aweigting checked
            if(this.checkBoxAweight.Checked == true && trArr != null)
            {
                myPane.YAxis.Title.Text = "Magnitude [db(A)]";
                 try
                {
                    trArr = doAweight(trArr);
               
                    myCurveTraffic.Label.IsVisible = false;
                    myCurveTraffic.Clear();
                    myCurveTraffic = myPane.AddCurve(trString,
                        null, trArr, Color.Blue, ZedGraph.SymbolType.None);

                    if (this.checkBoxTrStat.Checked == true)
                    {
                        myCurveTrStdDev.Clear();
                        myCurveTrStdDev.Label.IsVisible = false;
                        this.checkBoxTrStat_CheckedChanged(true, null);
                    }

                    zgc.AxisChange();
                    this.Show();
                    this.Refresh();
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }
            }

            /**

            if(this.checkBoxAweight.Checked == true && faArr != null)
            {
                myPane.YAxis.Title.Text = "Magnitude [db(A)]";
                 try
                {
                    faArr = doAweight(faArr);

                    myCurveFacade.Label.IsVisible = false;
                    myCurveFacade.Clear();
                    myCurveFacade = myPane.AddCurve("Level difference",
                        null, faArr, Color.Red, ZedGraph.SymbolType.None);
            
                    if (this.checkBoxFaStat.Checked == true)
                    {
                        myCurveFaStdDev.Clear();
                        myCurveFaStdDev.Label.IsVisible = false;
                        this.checkBoxFaStat_CheckedChanged(true, null);
                    }

                    zgc.AxisChange();
                    this.Show();
                    this.Refresh();
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }
            }
            
            if(this.checkBoxAweight.Checked == true && indoor != null)
            {
                try
                {
                    indoor = doAweight(indoor);

                    this.myCurveIndoor.Clear();
                    this.myCurveIndoor.Label.IsVisible = false;
                    myCurveIndoor = myPane.AddCurve("Indoor level",
                         null, indoor, Color.DarkOrchid, ZedGraph.SymbolType.None);

                    zgc.AxisChange();
                    this.Show();
                    this.Refresh();
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }
            
            }
             *  * **/
            #endregion

            #region Aweigting unchecked
            if (trArr != null && this.checkBoxAweight.Checked == false)
            {
                myPane.YAxis.Title.Text = "Magnitude [dB]";
                try
                {
                    trArr = doInverseAweight(trArr);

                    myCurveTraffic.Label.IsVisible = false;
                    myCurveTraffic.Clear();
                    myCurveTraffic = myPane.AddCurve(trString,
                        null, trArr, Color.Blue, ZedGraph.SymbolType.None);

                    if (this.checkBoxTrStat.Checked == true)
                    {
                        myCurveTrStdDev.Clear();
                        myCurveTrStdDev.Label.IsVisible = false;
                        this.checkBoxTrStat_CheckedChanged(true, null);
                    }

                    zgc.AxisChange();
                    this.Show();
                    this.Refresh();
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }
            }

            /**
            if (this.checkBoxAweight.Checked == false && faArr != null)
            {
                myPane.YAxis.Title.Text = "Magnitude [dB]";
                try
                {
                    faArr = doInverseAweight(faArr);

                    myCurveFacade.Label.IsVisible = false;
                    myCurveFacade.Clear();
                    myCurveFacade = myPane.AddCurve("Level difference",
                        null, faArr, Color.Red, ZedGraph.SymbolType.None);

                    if (this.checkBoxFaStat.Checked == true)
                    {
                        myCurveFaStdDev.Clear();
                        myCurveFaStdDev.Label.IsVisible = false;
                        this.checkBoxFaStat_CheckedChanged(true, null);
                    }

                    zgc.AxisChange();
                    this.Show();
                    this.Refresh();
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }
            }
             * 
             * **/
            if (this.checkBoxAweight.Checked == true && indoor != null)
            {
                try
                {
                    indoor = doInverseAweight(indoor);

                    this.myCurveIndoor.Clear();
                    this.myCurveIndoor.Label.IsVisible = false;
                    myCurveIndoor = myPane.AddCurve("Indoor level etter A-veiing",
                        null, indoor, Color.DarkOrchid, ZedGraph.SymbolType.None);

                    zgc.AxisChange();
                    this.Show();
                    this.Refresh();
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }

            }
            #endregion
        }
        #endregion 

        #region shows the standard deviation of the traffic data on the traffic graph
        private void checkBoxTrStat_CheckedChanged(object sender, EventArgs e)
        {
            if (this.checkBoxTrStat.Checked == true && this.trArr != null && this.trStdDev != null)
            {
                Console.WriteLine("Showing traffic data statistics");

                PointPairList hList = new PointPairList();

                for (int i = 0; i < trStdDev.Length; i++)
                {
                    try
                    {
                        double x = (double)this.trArr[i];
                        double hi = x - (this.trStdDev[i] / 2);
                        double low = x + (this.trStdDev[i] / 2);
                        hList.Add(x, hi, low);
                    }
                    catch (Exception exp)
                    {
                        Console.WriteLine(exp.Message);
                    }
                }

                myCurveTrStdDev = myPane.AddErrorBar("Traffic data std.dev.", hList, Color.Blue);
                myCurveTrStdDev.Bar.PenWidth = 3;
                myCurveTrStdDev.Bar.Symbol.IsVisible = false;
                this.Refresh();
                
            }
            else if (this.checkBoxTrStat.Checked == false && this.trArr != null && this.trStdDev != null)
            {
                myCurveTrStdDev.Clear();
                myCurveTrStdDev.Label.IsVisible = false;
                this.Refresh();
            }
            else if (this.trArr != null && this.trStdDev == null)
            {
                MessageBox.Show("No statistics available");
            }
            else
            {
                MessageBox.Show("Traffic data data missing");
            }
        }
        #endregion

        #region shows the standard deviation of the facade data on the facade graph
        private void checkBoxFaStat_CheckedChanged(object sender, EventArgs e)
        {
        double[] faStatMin = new Double[21];
        double[] faStatMax = new Double[21];
            
            if (this.checkBoxFaStat.Checked == true && this.faArr != null && this.faStdDev != null)
            {
                //Console.WriteLine("Showing facade data statistics");
                PointPairList hList = new PointPairList();
                for (int i = 0; i < faStdDev.Length; i++)
                {
                    double x = (double)this.faArr[i];
                    double hi = x - (this.faStdDev[i] / 2);
                    double low = x + (this.faStdDev[i] / 2);
                    hList.Add(x, hi, low);
                    //statstddev + and -
                    //faStatMax[i] = x + this.faStdDev[i];
                    //faStatMin[i] = x - this.faStdDev[i];
                }
                myCurveFaStdDev = myPane.AddErrorBar("Facade data std.dev.", hList, Color.Red);
                myCurveFaStdDev.Bar.PenWidth = 3;
                myCurveFaStdDev.Bar.Symbol.IsVisible = false;

                this.Refresh();
            }
            else if (this.checkBoxFaStat.Checked == false && this.faArr != null && this.faStdDev != null)
            {
                myCurveFaStdDev.Clear();
                myCurveFaStdDev.Label.IsVisible = false;
                this.Refresh();
            }
            else if (this.faArr != null && this.faStdDev == null)
            {
                MessageBox.Show("No statistics available");
            }
            else
            {
                MessageBox.Show("Facade data missing");
            }
        }
        #endregion

        #region smooothens the graphs
        private void checkBoxSmoothTr_CheckedChanged(object sender, EventArgs e)
        {
            //Checks if traffic data is available 
            //and then smoothens the traffic graph
            if (trArr != null && checkBoxSmooth.Checked == true)
            {
                myCurveTraffic.Line.IsSmooth = true;
            }
            else if (trArr != null && checkBoxSmooth.Checked == false)
            {
                myCurveTraffic.Line.IsSmooth = false;
            }
            else
            {
                //this.checkBoxSmooth.Enabled = false;
            }

            //checks if traffic data is available
            //and then smoothens the facade graph
            if (faArr != null && checkBoxSmooth.Checked == true)
            {
                myCurveFacade.Line.IsSmooth = true;
            }
            else if (faArr != null && checkBoxSmooth.Checked == false)
            {
                myCurveFacade.Line.IsSmooth = false;
            }
            else
            {
                //this.checkBoxSmooth.Enabled = false;
            }

            //checks if inddorLevel is checked, then smoothens the indoor graph
            if (this.checkBoxIndoorLevel.Checked == true && checkBoxSmooth.Checked == true)
            {
                this.myCurveIndoor.Line.IsSmooth = true;
            }
            else if (this.checkBoxIndoorLevel.Checked == true && checkBoxSmooth.Checked == false)
            {
                this.myCurveIndoor.Line.IsSmooth = false;
            }
            else
            {
            }
            this.Refresh();
        }
        #endregion

        #region removes legend from graph
        private void checkBoxLegend_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxLegend.Checked == true)
            {
                myPane.Legend.IsVisible = true;
                zgc.AxisChange();
                this.Show();
                this.Refresh();

            }
            else
            {
                myPane.Legend.IsVisible = false;
                zgc.AxisChange();
                this.Show();
                this.Refresh();
            }
        }
        #endregion

        #region getting statistics
        public double[] getStat()
        {
            return returns;
        }
        #endregion

        #region Displays the insulation curve
        private void checkBoxInsulDef_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxInsulDef.Checked == true && faArr != null)
            {
                this.button3.Enabled = true;
            }
            else
            {
                this.button3.Enabled = false;
            }
        }

        private void button3_Click(object sender, EventArgs e)
        {
            int i = 9; //9:insul
            Wall wall = new Wall(i, null, null, null, faArr, null);
            wall.Show();
        }
        #endregion

        #region Displays critical resonances
        private void checkBoxCritRes_CheckedChanged(object sender, EventArgs e)
        {

            Console.WriteLine("Adding graphics");

            if (fc1 != 0 && this.checkBoxCritRes.Checked == true)
            {
                double fc1nValue = 0;
                Console.WriteLine("Adding text to show critical frequency1 @" + fc1.ToString());
                // Add a text item to decorate the graph
                fc1nValue = faArr[Convert.ToInt32(fc1)];
                text = new TextObj("Critical\n frequency", (float)fc1, (float)fc1nValue-5);
                // Align the text such that the Bottom-Center is at (175, 80) in user scale coordinates
                // rotate the text 90 degrees
                text.FontSpec.Angle = 90;
                text.FontSpec.FontColor = Color.Red;
                // Align the text such that the Right-Center is at (700, 50) in user scale coordinates
                text.Location.AlignH = AlignH.Right;
                text.Location.AlignV = AlignV.Center;
                // Disable the border and background fill options for the text
                text.FontSpec.Fill.IsVisible = false;
                text.FontSpec.Border.IsVisible = false;
                myPane.GraphObjList.Add(text);
                text.IsClippedToChartRect = true;

                // Add an arrow pointer for the above text item
                arrow = new ArrowObj((float)fc1, (float)fc1nValue - 5, (float)fc1, (float)fc1nValue);
                arrow.Color = Color.Red;
                arrow.Location.CoordinateFrame = CoordType.AxisXYScale;
                myPane.GraphObjList.Add(arrow);
                arrow.IsClippedToChartRect = true;

            }
            if (fc1 != 0 && this.checkBoxCritRes.Checked == false)
            {
                text.IsVisible = false;
                arrow.IsVisible = false;
            }

            if (fc2 != 0 && this.checkBoxCritRes.Checked == true)
            {
                double fc2nValue = 0;
                Console.WriteLine("Adding text to show critical frequency1 @" + fc2.ToString());
                // Add a text item to decorate the graph
                fc2nValue = faArr[Convert.ToInt32(fc2)];
                text2 = new TextObj("Critical\n frequency", (float)fc2, (float)fc2nValue-5);
                // Align the text such that the Bottom-Center is at (175, 80) in user scale coordinates
                // rotate the text 90 degrees
                text2.FontSpec.Angle = 90;
                text2.FontSpec.FontColor = Color.Red;
                // Align the text such that the Right-Center is at (700, 50) in user scale coordinates
                text2.Location.AlignH = AlignH.Right;
                text2.Location.AlignV = AlignV.Center;
                // Disable the border and background fill options for the text
                text2.FontSpec.Fill.IsVisible = false;
                text2.FontSpec.Border.IsVisible = false;
                myPane.GraphObjList.Add(text2);
                text2.IsClippedToChartRect = true;

                // Add an arrow pointer for the above text item
                arrow2 = new ArrowObj((float)fc2, (float)fc2nValue - 5, (float)fc2, (float)fc2nValue );
                arrow2.Color = Color.Red;
                arrow2.Location.CoordinateFrame = CoordType.AxisXYScale;
                myPane.GraphObjList.Add(arrow2);
                arrow2.IsClippedToChartRect = true;

            }
            if (fc2 != 0 && this.checkBoxCritRes.Checked == false)
            {
                text2.IsVisible = false;
                arrow2.IsVisible = false;
            }


            if (fd != 0 && this.checkBoxCritRes.Checked == true)
            {
                Console.WriteLine("Adding text to show mass-air-mass resonance");
                double fdnValue = 0;
                Console.WriteLine("Adding text to show mass-air-mass resoanance @ " + fd.ToString());
                // Add a text item to decorate the graph
                fdnValue = faArr[Convert.ToInt32(fd)];
                text3 = new TextObj("Mas-air-mass\n resonance", (float)fd, (float)fdnValue-10);

                // Align the text such that the Bottom-Center is at (175, 80) in user scale coordinates
                // rotate the text 90 degrees
                text3.FontSpec.Angle = 90;
                text3.FontSpec.FontColor = Color.Red;
                // Align the text such that the Right-Center is at (700, 50) in user scale coordinates
                text3.Location.AlignH = AlignH.Right;
                text3.Location.AlignV = AlignV.Center;
                // Disable the border and background fill options for the text
                text3.FontSpec.Fill.IsVisible = false;
                text3.FontSpec.Border.IsVisible = false;
                myPane.GraphObjList.Add(text3);
                text3.IsClippedToChartRect = true;

                // Add a BoxObj to show a colored band behind the graph data
                //19, -10000, 19, 10000
                box = new BoxObj(fd-(1+0.23156), 100, 3, 100,
                      Color.Empty, Color.FromArgb(225, 245, 225));
                box.Location.CoordinateFrame = CoordType.AxisXYScale;
                // Align the left-top of the box to (0, 110)
                box.Location.AlignH = AlignH.Left;
                box.Location.AlignV = AlignV.Top;
                // place the box behind the axis items, so the grid is drawn on top of it
                box.ZOrder = ZOrder.E_BehindAxis;
                myPane.GraphObjList.Add(box);
                box.IsClippedToChartRect = true;

                this.Refresh();
            }
            if (fd != 0 && this.checkBoxCritRes.Checked == false)
            {
                box.IsVisible = false;
                text3.IsVisible = false;
            }
            if (fc1 == 0)
            {
                MessageBox.Show("Window calculations must be set");
            }

            this.Refresh();
        }
        #endregion 

        #region Displays Traffic peaks
        private void checkBoxCritTopsTr_CheckedChanged(object sender, EventArgs e)
        {
            int freqTemp = 0;
            int fdn = 0;
            if (opm != 0 && this.sylnr != 0 && this.checkBoxCritTopsTr.Checked == true)
            {
                freqTemp = (opm * sylnr / 2) / 60;
                double delta = 0;
                double deltaf = 0;
                delta = 0.23156;

                try
                {
                    for (int n = 0; n < trArr.Length; n++)
                    {
                        deltaf = delta * freqband[n] / 2;
                        //Console.WriteLine("Finding the resonannce @ " + n.ToString() + delta.ToString() + " and deltaf" + deltaf.ToString());
                        if ((freqTemp > (freqband[n] - deltaf)) && (freqTemp < (freqband[n] + deltaf)))
                        {
                            fdn = n +1;
                            Console.WriteLine("traffic peak @ " + fdn.ToString());
                        }
                    }
                
                
                double fdnValue = 0;
                // Add a text item to decorate the graph
                fdnValue = trArr[Convert.ToInt32(fdn)];
                text4 = new TextObj("Traffic\n peak", (float)fdn, (float)fdnValue - 10);

                // Align the text such that the Bottom-Center is at (175, 80) in user scale coordinates
                // rotate the text 90 degrees
                text4.FontSpec.Angle = 90;
                text4.FontSpec.FontColor = Color.Blue;
                // Align the text such that the Right-Center is at (700, 50) in user scale coordinates
                text4.Location.AlignH = AlignH.Right;
                text4.Location.AlignV = AlignV.Center;
                // Disable the border and background fill options for the text
                text4.FontSpec.Fill.IsVisible = false;
                text4.FontSpec.Border.IsVisible = false;
                myPane.GraphObjList.Add(text4);
                text4.IsClippedToChartRect = true;

                // Add a BoxObj to show a colored band behind the graph data
                //19, -10000, 19, 10000
                box2 = new BoxObj(fdn - (1 + 0.23156), 100, 2.7, 100,
                      Color.Empty, Color.FromArgb(180, 245, 225));
                box2.Location.CoordinateFrame = CoordType.AxisXYScale;
                // Align the left-top of the box to (0, 110)
                box2.Location.AlignH = AlignH.Left;
                box2.Location.AlignV = AlignV.Top;
                // place the box behind the axis items, so the grid is drawn on top of it
                box2.ZOrder = ZOrder.E_BehindAxis;
                myPane.GraphObjList.Add(box2);
                box2.IsClippedToChartRect = true;

                this.Refresh();

            }
                           catch (Exception exp)
                {
                    Console.WriteLine(exp.Message);
                }



            }
            else if ( this.checkBoxCritTopsTr.Checked == false)
            {
                try
                {
                    box2.IsVisible = false;
                    text4.IsVisible = false;
                }
                catch (Exception exp)
                {
                    Console.WriteLine(exp.Message);
                }
            }
            else
            {
                MessageBox.Show("Traffic calculations must be defined");
            }

            this.Refresh();
        }
        #endregion

    }
}