 function [Par,Parw,Avvik,Refs,f]=luftlyd(L1,L2,B2,T2,S,V,T0,P)
%Utregning av veid luftlydreduksjon basert p mlte verdier
%
%   Bruk:
%   [Par,Parw,Avvik,Refs,f]=luftlyd(L1,L2,B2,T2,S,V,T0,P)
%
%   L1,L2,B2 and T2 m vre gyldig fra enten 50-5000 eller 100-3150 Hz i 1/3 oktav middelverdier
%   L1 er niv i senderrom
%   L2 er niv i mottakerrom
%   B2 er bakgrunnsstyniv i mottakerrom
%   T2 er etterklangstid i mottakerrom
%   S er skilleflatearealet
%   V er volum til mottakerrom
%   T0 er referanse-etterklangstid. Satt til standardverdi for stuer; 0,5 sekunder hvis ikke spesifisert
%   P angir parameter  regne ut. P=0: R'w, P=1: Dw, P=2: Dnw, P=3: DnTw, P=4: Dts,2m
%   Par er beregnet parameter (R', D, Dn, DnT eller Dts,2m )
%   Parw er beregnet veid parameter (R'w, Dw, Dnw, DnTw eller Dts,2m)
%   Avvik er avvik fra referansekurven fra 100 til 3150 Hz
%   Refs er hevet/senket referansekurve
%   f er 1/3 oktav-frekvensene fra 50 til 5000 Hz
%
%Beregnet ut fra ISO 140-4:1997 og ISO 717-1:1997
%
% Jens Jrgen Dammerud 2003

%Sjekker lengden p niv- og etterklangsmatrisene
L2=L2';
T2=T2';
sL1=size(L1);
sL2=size(L2);
sB2=size(B2);
sT2=size(T2);
if sL1(2)>1
   L1=L1';
elseif sL2(2)>1
   L2=L2';
elseif sB2(2)>1
   B2=B2';
elseif sT2(2)>1
   T2=T2';
end

%Gir advarsel dersom skilleflateareal er under 10 kvadratmeter
if S<10
   disp('Advarsel! Husk at dersom S<10, sett S til maks(S,V/7.5) eller lik 10 kvadratmeter.')
end



if length(L1) == 21 & length(L2) == 21 & length(B2) == 21 & length(T2) == 21
   disp('Datasett fra 50-5000 Hz detektert')
elseif length(L1) == 16 & length(L2) == 16 & length(B2) == 16 & length(T2) == 16
   disp('Datasett fra 100-3150 Hz detektert')
elseif length(L1) == 16 & length(L2) == 16 & length(B2) == 16 & length(T2) == 16
   disp('Datasett fra 100-3150 Hz detektert')
else
   disp('Ugyldig datasett detektert. Beregning avbrutt.')
   Par=NaN;Parw=NaN;Avvik=NaN;Refs=NaN;f=NaN;
   return
end


%Definere referansekurver og frekvensaksen
%ref=[NaN NaN NaN 33 36 39 42 45 48 51 52 53 54 55 56 56 56 56 56 NaN NaN];
%C503150ref=[-40 -36 -33 -29 -26 -23 -21 -19 -17 -15 -13 -12 -11 -10 -9 -9 -9 -9 -9 NaN NaN];
%C505kref=[-41 -37 -34 -30 -27 -24 -22 -20 -18 -16 -14 -13 -12 -11 -10 -10 -10 -10 -10 -10 -10];
%C1005kref=C505kref;
%Cref=C503150ref;
%Ctrref=[-25 -23 -21 -20 -20 -18 -16 -15 -14 -13 -12 -11 -9 -8 -9 -10 -11 -13 -15 -16 -18];
%f=[50 63 80 100 125 160 200 250 315 400 500 630 800 1000 1250 1600 2000 2500 3150 4000 5000];

%Definere referansekurver og frekvensaksen
ref=[NaN NaN NaN 33 36 39 42 45 48 51 52 53 54 55 56 56 56 56 56 NaN NaN];
C503150ref=[-40 -36 -33 -29 -26 -23 -21 -19 -17 -15 -13 -12 -11 -10 -9 -9 -9 -9 -9 NaN NaN];
C635kref=[-37 -34 -30 -27 -24 -22 -20 -18 -16 -14 -13 -12 -11 -10 -10 -10 -10 -10 -10 -10];
C1005kref=C635kref;
Cref=C503150ref;
Ctrref=[-23 -21 -20 -20 -18 -16 -15 -14 -13 -12 -11 -9 -8 -9 -10 -11 -13 -15 -16 -18];
f=[50 63 80 100 125 160 200 250 315 400 500 630 800 1000 1250 1600 2000 2500 3150 4000 5000];


%Sjekker margin for bakgrunnssty
bn=0;
for i=1:20,
   if L2(i)-B2(i) <= 10 & L2(i)-B2(i) >= 6 %Korrigerer dersom margin er 6 - 10 dB
      L2(i)=10*log10(10^(L2(i)/10)-10^(B2(i)/10));
      s=sprintf('disp(''Mottakerromniv korrigert for bakgrunnssty ved %d Hz.'')',f(i));
      eval(s)
   elseif L2(i)-B2(i)< 6 %Klarer ikke  korrigere dersom margin er under 6 dB
      s=sprintf('disp(''Bakgrunsstyniv ved %d Hz for hy. Ikke i stand til  korrigere for dette.'')',f(i));
      eval(s)
      bn=bn+1;
   end
end
if bn>0 %Varsler dersom margin er for lav
   disp(' ')
   disp('Beregnede parametre er ikke plitelige p grunn av hyt bakgrunnsstyniv.')
end
         
if P==0 %R'
   A=24*log(10)/343.*V./T2;
   R=L1-L2+10*log10(S./A)-3;
   p=R;
elseif P==1 %D
   D=L1-L2;
   p=D;
elseif P==2 %Dn %trukket fra 3dB p utenivet i ligningen under for  f lydnivet 2meter fra veggen!!!
   A=24*log(10)/343.*V./T2;
   D=L1-L2;
   A0=10; %A0=0.32*V;
   Dn=D-3-10*log10(A./A0);
   p=Dn;
elseif P==3 %DnT
   %T0=0.5;
   D=L1-L2;
   DnT=D+10*log10(T2./T0);
   p=DnT;
elseif P==4 %Dls2m trukket fra 3dB p utenivet i ligningen under for  f lydnivet 2meter fra veggen!!!
   %T0=0.5;
   D=L1-L2;
   Dls2m=D+10*log10(T2./T0);
   %Dls2m=D-3+10*log10(T2./T0);
   p=Dls2m;
end

avvik=0;
for j=4:19, %Finner totalt avvik relativt referansekurven
   if p(j)<ref(j)
      avvik=avvik+ref(j)-p(j);
      disp(avvik);
   end
end

if round(avvik) > 32 %Dersom avvik er over 32 dB
   c=0;
   while round(avvik) > 32 %Senker referansekurve til avvik er lik eller under 32 dB
      avvik=0;
      c=c+1;
      for j=4:19,
         if p(j)<ref(j)-c

            avvik=avvik+ref(j)-c-p(j);
         end
     end
   end
   pw=52-c;
elseif round(avvik) < 32 %Dersom avvik er under 32 dB
   c=0;
   while round(avvik) < 32 %Hever referanskurve til avvik er over 32 dB
      avvik=0;
      c=c-1;
      for j=4:19,
         if p(j)<ref(j)-c
            avvik=avvik+ref(j)-c-p(j);
         end
      end
   end
   if avvik > 32 %Dersom avvik ble over 32 dB, m referanse kurve senkes 1 dB
      c=c+1;
   end
   pw=52-c;
end
Par=p;
Parw=pw;
Refs=ref-c;
Avvik=zeros(16,1);
for i=4:19,
   Avvik(i-3)=p(i)-(ref(i)-c);
end

return


