function plotluftlyd(Par,Refs,f,Pp)
%Plotter resultat fra beregning av luftlydreduksjon
%
%   Bruk:
%   plotluftlyd(Par,Refs,f,Pp)
%
%   Par er parameter som skal plottes
%   Refs er hevet/senket referansekurve
%   f er 1/3 oktav-frekvensene fra 50 til 5000 Hz
%   Pp er parameter som skal plottes. Pp=0: R'w, Pp=1: Dw, Pp=2: Dnw, Pp=3: DnTw, Pp=4: Dts,2m
%
%Plottet ut fra ISO 717-1:1997
%
% Jens Jrgen Dammerud 2003

%Oppretter figur ut fra hvilke parameter som skal plottes (R', D, Dn, DnTw eller Dts,2m)
if Pp==0
   %Setter min og maks til y-akse ut fra min og maks til R'
   dBmin=10;
   dBmax=60+dBmin;
   no=6;
   if min(Par)<10
      dBmin=0;
      no=no+1;
   end
   if max(Par)>70
      dBmax=80;
      no=no+1;
   elseif max(Par)>80
      dBmax=90;
      no=no+1;
   end
   figure('Position',[100 100 500 500*(13.5/11.2)/6*no*(13.5/11.2)/(14.65/11.5)],'Color',[1 1 1]);
   semilogx(f,Par)
   hold on
   xlabel('Frekvenst [Hz]')%,'FontWeight','bold','FontSize',8)
   ylabel('Lydreduksjonstall R'' [dB]')%,'FontWeight','bold','FontSize',8)
   title('Lydreduksjonstall i henhold til ISO 140-4','FontWeight','normal','FontSize',11)
   set(gca,'FontSize',8)
elseif Pp==1
   %Setter min og maks til y-akse ut fra min og maks til R'
   dBmin=10;
   dBmax=60+dBmin;
   no=6;
   if min(Par)<10
      dBmin=0;
      no=no+1;
   end
   if max(Par)>70
      dBmax=80;
      no=no+1;
   elseif max(Par)>80
      dBmax=90;
      no=no+1;
   end
   figure('Position',[100 100 500 500*(13.5/11.2)/6*no*(13.5/11.2)/(14.65/11.5)],'Color',[1 1 1]);
   semilogx(f,Par)
   hold on
   xlabel('Frekvens [Hz]');%,'FontWeight')%,'bold','FontSize',8)
   ylabel('Nivforskjell D [dB]')%,'FontWeight')%,'bold','FontSize',8)
   title('Nivforskjell i henhold til ISO 140-4','FontWeight','normal','FontSize',11)
   set(gca,'FontSize',8)
elseif Pp==2
   %Setter min og maks til y-akse ut fra min og maks til R'
   dBmin=10;
   dBmax=60+dBmin;
   no=6;
   if min(Par)<10
      dBmin=0;
      no=no+1;
   end
   if max(Par)>70
      dBmax=80;
      no=no+1;
   elseif max(Par)>80
      dBmax=90;
      no=no+1;
   end
   figure('Position',[100 100 500 500*(13.5/11.2)/6*no*(13.5/11.2)/(14.65/11.5)],'Color',[1 1 1]);
   semilogx(f,Par)
   hold on
   xlabel('Frekvens [Hz]');%,'FontWeight'); %,'bold','FontSize',8)
   ylabel('Normalisert nivforskjell Dn [dB]');%,'FontWeight','bold','FontSize',8)
   title('Normalisert nivforskjell i henhold til ISO 140-4','FontWeight','normal','FontSize',11)
   set(gca,'FontSize',8)
elseif Pp==3
   %Setter min og maks til y-akse ut fra min og maks til R'
   dBmin=10;
   dBmax=60+dBmin;
   no=6;
   if min(Par)<10
      dBmin=0;
      no=no+1;
   end
   if max(Par)>70
      dBmax=80;
      no=no+1;
   elseif max(Par)>80
      dBmax=90;
      no=no+1;
   end
   figure('Position',[100 100 500 500*(13.5/11.2)/6*no*(13.5/11.2)/(14.65/11.5)],'Color',[1 1 1]);
   semilogx(f,Par)
   hold on
   xlabel('Frekvens [Hz]')%,'FontWeight','bold','FontSize',8)
   ylabel('Standarisert nivforskjell DnTw [dB]')%,'FontWeight','bold','FontSize',8)
   title('Standarisert nivforskjell i henhold til ISO 140-4','FontWeight','normal','FontSize',11)
   set(gca,'FontSize',8)
elseif Pp==4
   %Setter min og maks til y-akse ut fra min og maks til R'
   dBmin=10;
   dBmax=60+dBmin;
   no=6;
   if min(Par)<10
      dBmin=0;
      no=no+1;
   end
   if max(Par)>70
      dBmax=80;
      no=no+1;
   elseif max(Par)>80
      dBmax=90;
      no=no+1;
   end
   figure('Position',[100 100 500 500*(13.5/11.2)/6*no*(13.5/11.2)/(14.65/11.5)],'Color',[1 1 1]);
   semilogx(f,Par)
   hold on
   xlabel('Frekvens [Hz]','FontWeight','bold','FontSize',20)
   ylabel('Fasadenivforskjell Dls,2m [dB]','FontWeight','bold','FontSize',20)
   title('Fasadenivforskjell i henhold til ISO 140-5','FontWeight','normal','FontSize',24)
   set(gca,'FontSize',8)
end

%Definerer aksene
axis([50 5000 dBmin dBmax])

%Plotter vertikale hjelpelinjer
plot([100 100],[dBmin+0.8 dBmax-0.7],'k:','Color',[0.6 0.6 0.6])
plot([3150 3150],[dBmin+0.8 dBmax-0.7],'k:','Color',[0.6 0.6 0.6])

plot([63 63],[dBmin+0.8 dBmax-0.7],'Color',[0.6 0.6 0.6])
plot([125 125],[dBmin+0.8 dBmax-0.7],'Color',[0.6 0.6 0.6])
plot([250 250],[dBmin+0.8 dBmax-0.7],'Color',[0.6 0.6 0.6])
plot([500 500],[dBmin+0.8 dBmax-0.7],'Color',[0.6 0.6 0.6])
plot([1000 1000],[dBmin+0.8 dBmax-0.7],'Color',[0.6 0.6 0.6])
plot([2000 2000],[dBmin+0.8 dBmax-0.7],'Color',[0.6 0.6 0.6])
plot([4000 4000],[dBmin+0.8 dBmax-0.7],'Color',[0.6 0.6 0.6])

%Plotter horisontale hjelpelinjer
for i=1:dBmax/10-dBmin/10-1,
   plot([50 5000],[dBmax-i*10 dBmax-i*10],'Color',[0.6 0.6 0.6])
end

%Tegner merker p x-aksen
set(gca,'XTick',[50 63 80 100 125 160 200 250 315 400 500 630 800 1000 1250 1600 2000 2500 3150 4000 5000])
set(gca,'XTickLabel',{ '' '63' '' '' '125' '' '' '250' '' '' '500' '' '' '1000' '' '' '2000' '' '' '4000' ''},'FontSize',8)

%Plotter referansekurven og setter inn boks med forklaring av de to kurvene
if Pp==0
   plot(f,Refs,'r:')
   legend('R''', 'Ref-kurve',2);
elseif Pp==1
   plot(f,Refs,'r:')
   legend('D', 'Ref-kurve',2);
elseif Pp==2
   plot(f,Refs,'r:')
   legend('Dn', 'Ref-kurve',2);
elseif Pp==3
   plot(f,Refs,'r:')
   legend('DnT', 'Ref-kurve',2);
elseif Pp==4
   plot(f,Refs,'r:')
   legend('Dls,2m', 'Ref-kurve',2);
end

return
